`define N_EXT_ADDR 26

module socTop (
    pbRstN,
    poRstN,
    flashRstN,
    sysClk,
    rs232Atx,
    rs232Arx,
    extAddr,
    extData,
    flashHiCeN,
    flashLoCeN,
    flashWeN,
    flashOeN,
    sramCeN,
    sramBsN,
    sramWeN,
    sramOeN,
    ledOut,
    switchIn,
    UJTAG_UJTAG_TCK,
    UJTAG_UJTAG_TDI,
    UJTAG_UJTAG_TMS,
    UJTAG_UJTAG_TRSTB,
    UJTAG_UJTAG_TDO,
    JTCK,
    JTDI,
    JTMS,
    JRTCK,
    JTDO,
    sysClk_16MHz,
    pllLock,
    dummyIn,
    dummyOut
    ); /* synthesis syn_global_buffers = 10 */

// I/O definitions:

    // System clocks:
    input  sysClk;

    // Resets:
    input  pbRstN;                  // external reset connected to pushbutton
    input  poRstN;                  // external reset not connected to pushbutton
    output flashRstN;               // reset to FLASH

    // RS232 connections:
    output rs232Atx;                // serial transmit line
    input  rs232Arx;                // serial receive line

    // External FLASH/RAM signals:
    inout  [31:0] extData;          // bidirectional data to external devices
    output [`N_EXT_ADDR-1:2] extAddr;

    output flashHiCeN;
    output flashLoCeN;
    output flashWeN;
    output flashOeN;
    output sramCeN;
    output [3:0] sramBsN;           // SRAM Byte Select signals
    output sramWeN;
    output sramOeN;

    // Switches, LEDs
    output [9:0] ledOut;
    input  [9:0] switchIn;

    // JTAG for FlashPro3 connector
    input UJTAG_UJTAG_TCK;
    input UJTAG_UJTAG_TDI;
    input UJTAG_UJTAG_TMS;
    input UJTAG_UJTAG_TRSTB;
    output UJTAG_UJTAG_TDO;

    // ICE Debug connector
    input JTCK;
    input JTDI;
    input JTMS;
    output JRTCK;
    output JTDO;

    // Pll signals
    output sysClk_16MHz;
    output pllLock;

    // Dummy signals
    input  dummyIn;
    output dummyOut;

// Declarations:

    // Clocks, Resets
    wire sysClk_48MHz;
    wire sysClk_16MHz;
    wire poRstNi;
    wire pbRstNi;

    // APB slave interface
    wire HRESETN;

    wire [31:0] APBmslave0_PRDATA;
    wire [23:0] APBmslave0_PADDR;
    wire APBmslave0_PENABLE;
    wire APBmslave0_PSELx;
    wire [31:0] APBmslave0_PWDATA;
    wire APBmslave0_PWRITE;

    // Parallel Port signals
    wire [7:0] parDin;
    wire [7:0] parDout;
    wire parOeN;
    wire led9;
    wire led8;

    // ARM debugger
    wire nTDOEN;
    wire JTDOi;

    // External memories
    wire [27:0] memAddr;
    wire flashCeN;
    wire flashOeN;
    wire sramOeN;

    // Remap
    wire Remap;

// Assignments:

    assign rs232Atx = 1'b0;

    // Keep these from being unused inputs
    assign dummyOut = switchIn[8] & rs232Arx & dummyIn;

    // Connect LEDs & switches to Parallel Port
    assign ledOut = {led9, led8, parDout[7:0]};
    assign parDin = switchIn[7:0];

    // ICE debugger
    assign JTDO = nTDOEN ? 1'bZ : JTDOi;

    // External memories
    assign flashRstN = pbRstNi;
    assign flashHiCeN = flashCeN;
    assign flashLoCeN = flashCeN;
    assign extAddr[`N_EXT_ADDR-1:2] = memAddr[`N_EXT_ADDR-3:0];
    //assign memOeN = flashOeN & sramOeN;

    // Remap
    assign Remap = ~switchIn[9]; //Note that switchIn[9] governs remapping

// Instantiations:

    // Global resources for clock, reset
    actel_pll_48MHz_16MHz pll1 (.POWERDOWN(poRstNi), .CLKA(sysClk_48MHz), .LOCK(pllLock), .GLA(sysClk_16MHz));
    CLKINT ci0 (.A(sysClk),      .Y(sysClk_48MHz));
    CLKINT ci1 (.A(poRstN),      .Y(poRstNi));
    CLKINT ci2 (.A(pbRstN),      .Y(pbRstNi));

    // Instantiate ARM7 Subsystem:
    Example example0
    (
        // Inputs
        .APBmslave0_PRDATA      (APBmslave0_PRDATA),
        .NSYSRESET              (pbRstNi),
        .RV_ICE_If_RV_TCK       (JTCK),
        .RV_ICE_If_RV_TDI       (JTDI),
        .RV_ICE_If_RV_TMS       (JTMS),
        .RV_ICE_If_RV_nSRST_IN  (pbRstNi),
        .RV_ICE_If_RV_nTRST     (poRstNi),
        .Remap                  (Remap),
        .SYSCLK                 (sysClk_16MHz),
        .UJTAG_UJTAG_TCK        (UJTAG_UJTAG_TCK),
        .UJTAG_UJTAG_TDI        (UJTAG_UJTAG_TDI),
        .UJTAG_UJTAG_TMS        (UJTAG_UJTAG_TMS),
        .UJTAG_UJTAG_TRSTB      (UJTAG_UJTAG_TRSTB),
        // Outputs
        .APBmslave0_PADDR       (APBmslave0_PADDR),
        .APBmslave0_PENABLE     (APBmslave0_PENABLE),
        .APBmslave0_PSELx       (APBmslave0_PSELx),
        .APBmslave0_PWDATA      (APBmslave0_PWDATA),
        .APBmslave0_PWRITE      (APBmslave0_PWRITE),
        .ExternalMemoryInterface_FlashCSN    (flashCeN),  
        .ExternalMemoryInterface_FlashOEnN   (flashOeN),
        .ExternalMemoryInterface_FlashWEnN   (flashWeN),
        .ExternalMemoryInterface_MemAddr     (memAddr),
        .ExternalMemoryInterface_MemReadN    (),
        .ExternalMemoryInterface_MemWriteN   (),
        .ExternalMemoryInterface_SramByteN   (sramBsN),
        .ExternalMemoryInterface_SramCSN     (sramCeN),
        .ExternalMemoryInterface_SramClk     (),
        .ExternalMemoryInterface_SramOEnN    (sramOeN),
        .ExternalMemoryInterface_SramWEnN    (sramWeN),
        .ExternalMemoryInterface_MemData     (extData),
        .HRESETN                (HRESETN),
        .RV_ICE_If_RV_RTCK      (JRTCK),
        .RV_ICE_If_RV_TDOUT     (JTDOi),
        .RV_ICE_If_RV_nTDOEN    (nTDOEN),
        .UJTAG_UJTAG_TDO        (UJTAG_UJTAG_TDO)
    );

    // Instantiate Parallel Port (APB)
    socParPort socParPort0
    (
        .PRESETn    (HRESETN),
        .PCLK       (sysClk_16MHz),
        .PADDR      (APBmslave0_PADDR[3:2]),
        .PWDATA     (APBmslave0_PWDATA),
        .PWRITE     (APBmslave0_PWRITE),
        .PSEL       (APBmslave0_PSELx),
        .PENABLE    (APBmslave0_PENABLE),
        .PRDATA     (APBmslave0_PRDATA),

        .errorN     (1'b1),             // bootstrap inactive
        .slct       (1'b0),             // bootstrap inactive
        .pe         (1'b0),             // bootstrap inactive
        .ackN       (1'b1),             // bootstrap inactive
        .busyN      (1'b1),             // bootstrap inactive
        .bidEn      (1'b1),             // bootstrap high
        .intSel     (1'b1),             // bootstrap to force latched interrupt mode

        .blkInt     (),                 // unused
        .slctInN    (),                 // unused
        .init       (),                 // unused
        .autoFdXtN  (led9),             // unused
        .strobeN    (led8),             // unused

        .parDin      (parDin),
        .parDout     (parDout),
        .parOeN      (parOeN)
    );

endmodule
